==> hlog-0.21-rc1.tar.gz: hlog-0.21-rc1/daemon/daemon.c <==
/*
 * Copyright (c) 2020-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <locale.h>

#include <jeffpc/version.h>
#include <jeffpc/error.h>
#include <jeffpc/io.h>
#include <jeffpc/thread.h>

#include <hlog/country.h>
#include <hlog/version.h>
#include <hlog-rpc/announce.h>
#include <hlog-rpc/announce-pub.h>

#include "index.h"
#include "location.h"
#include "rig.h"
#include "wsjtx.h"

#define DEFAULT_RIG_HOST	"localhost"
#define DEFAULT_RIG_PORT	4532

#define DATA_DIR_COUNTRY_FNAME	"cty.csv"

static int load_countries(void)
{
	int ret;

	cmn_err(CE_INFO, "Loading countries data from %s...",
		DATA_DIR_COUNTRY_FNAME);
	ret = country_load(AT_FDCWD, DATA_DIR_COUNTRY_FNAME);
	cmn_err(CE_INFO, "%s.", ret ? "error" : "done");

	if (ret)
		cmn_err(CE_ERROR, "Error: failed to load country data: %s",
			xstrerror(ret));

	return ret;
}

static int load_databases(void)
{
	int ret;

	/* first, load the required ones */
	ret = load_countries();

	return ret;
}

static void usage(const char *prog, const char *fmt, ...)
{
	if (fmt) {
		va_list ap;

		fprintf(stderr, "Error: ");

		va_start(ap, fmt);
		vfprintf(stderr, fmt, ap);
		va_end(ap);

		fprintf(stderr, "\n");
	}

#define P(...) fprintf(stderr, __VA_ARGS__)

	P("hlogd version %s\n\n", version_string);
	P("Usage: %s [-R <hostname>] [-p <port>] [-m <model>] [-r <device>] "
	  "<data dir>\n", prog);
	P("\n");
	P("Options:\n");
	P("  -R <hostname>  - Flex rig hostname to listen on (default: %s)\n",
	  DEFAULT_RIG_HOST);
	P("  -p <port>      - Flex rig TCP port to listen on (default: %u)\n",
	  DEFAULT_RIG_PORT);
	P("  -m <model>     - hamlib rig model\n");
	P("  -r <device>    - hamlib rig device\n");
	P("\n");
	P("If neither the hamlib device nor the model are not specified, rig\n"
	  "control will not be available.\n");
	P("\n");
	P("If neither Flex rig hostname nor port are specified, the "
	  "defaults are ignored\n");
	P("and rig control services are not provided.\n");

#undef P

	exit(1);
}

int main(int argc, char **argv)
{
	const char *data_dir;
	const char *rig_host;
	const char *rig_dev;
	uint32_t rig_model;
	uint16_t rig_port;
	int opt;
	int ret;

	setlocale(LC_ALL, "");
	ASSERT0(putenv("TZ=UTC"));

	xthr_set_name("main");

	rig_host = NULL;
	rig_dev = NULL;
	rig_model = 0;
	rig_port = 0;
	while ((opt = getopt(argc, argv, "R:p:m:r:")) != -1) {
		switch (opt) {
			case 'R':
				if (rig_host)
					usage(argv[0], "hamlib rig hostname "
					      "specified multiple times");

				rig_host = optarg;
				break;
			case 'p':
				if (rig_port)
					usage(argv[0], "hamlib rig port "
					      "specified multiple times");

				ret = str2u16(optarg, &rig_port);
				if (ret)
					usage(argv[0], "invalid hamlib rig "
					      "port: %s", xstrerror(ret));

				if (!rig_port)
					usage(argv[0], "invalid hamlib rig "
					      "port: zero port number");
				break;
			case 'm':
				if (rig_model)
					usage(argv[0], "hamlib rig model "
					      "specified multiple times");

				ret = str2u32(optarg, &rig_model);
				if (ret)
					usage(argv[0], "invalid hamlib rig "
					      "model: %s", xstrerror(ret));

				if (!rig_model)
					usage(argv[0], "invalid hamlib rig "
					      "model: zero model number");
				break;
			case 'r':
				if (rig_dev)
					usage(argv[0], "hamlib rig device "
					      "specified multiple times");

				rig_dev = optarg;
				break;
			default:
				usage(argv[0], NULL);
				break;
		}
	}

	/* missing data dir path */
	if ((optind + 1) != argc)
		usage(argv[0], "data directory is required");

	data_dir = argv[argc - 1];

	/*
	 * make sure either both rig model and device are specified, or
	 * neither is specified
	 */
	if ((rig_model || rig_dev) && !(rig_model && rig_dev))
		usage(argv[0], "rig model & device must be both specified");

	/*
	 * make sure both rig host and port are both set if at least one of
	 * them is specified
	 */
	if (rig_host || rig_port) {
		rig_host = rig_host ? rig_host : DEFAULT_RIG_HOST;
		rig_port = rig_port ? rig_port : DEFAULT_RIG_PORT;
	}

	cmn_err(CE_INFO, "hlogd version %s", version_string);
	cmn_err(CE_INFO, "libjeffpc version %s", jeffpc_version);

	/*
	 * chdir to the data directory to make our lives in the future much
	 * easier.  We will be able to use relative paths everywhere.
	 */
	if (chdir(data_dir)) {
		cmn_err(CE_ERROR, "Error: failed to change to data dir: %s",
			strerror(errno));
		goto err;
	}

	ret = load_databases();
	if (ret)
		goto err;

	ret = index_init();
	if (ret)
		goto err;

	ret = announce_start();
	if (ret)
		goto err;

	ret = location_start();
	if (ret)
		goto err_announce;

	ret = rig_start(rig_model, rig_dev, rig_host, rig_port);
	if (ret)
		goto err_location;

	ret = wsjtx_start();
	if (ret)
		goto err_rig;

	for (;;)
		sleep(60);

	wsjtx_stop();
	rig_stop();
	location_stop();
	announce_stop();

	return 0;

err_rig:
	rig_stop();

err_location:
	location_stop();

err_announce:
	announce_stop();

err:
	return 1;
}

==> hlog-0.22.tar.gz: hlog-0.22/daemon/daemon.c <==
/*
 * Copyright (c) 2020-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <locale.h>

#include <jeffpc/version.h>
#include <jeffpc/error.h>
#include <jeffpc/io.h>
#include <jeffpc/thread.h>

#include <hlog/country.h>
#include <hlog/version.h>
#include <hlog-rpc/announce.h>
#include <hlog-rpc/announce-pub.h>

#include "index.h"
#include "location.h"
#include "rig.h"
#include "wsjtx.h"

#define DEFAULT_RIG_HOST	"localhost"
#define DEFAULT_RIG_PORT	4532

#define DATA_DIR_COUNTRY_FNAME	"data/cty.csv"

static int load_countries(void)
{
	int ret;

	cmn_err(CE_INFO, "Loading countries data from %s...",
		DATA_DIR_COUNTRY_FNAME);
	ret = country_load(AT_FDCWD, DATA_DIR_COUNTRY_FNAME);
	cmn_err(CE_INFO, "%s.", ret ? "error" : "done");

	if (ret)
		cmn_err(CE_ERROR, "Error: failed to load country data: %s",
			xstrerror(ret));

	return ret;
}

static int load_databases(void)
{
	int ret;

	/* first, load the required ones */
	ret = load_countries();

	return ret;
}

static void usage(const char *prog, const char *fmt, ...)
{
	if (fmt) {
		va_list ap;

		fprintf(stderr, "Error: ");

		va_start(ap, fmt);
		vfprintf(stderr, fmt, ap);
		va_end(ap);

		fprintf(stderr, "\n");
	}

#define P(...) fprintf(stderr, __VA_ARGS__)

	P("hlogd version %s\n\n", version_string);
	P("Usage: %s [-R <hostname>] [-p <port>] [-m <model>] [-r <device>] "
	  "<data dir>\n", prog);
	P("\n");
	P("Options:\n");
	P("  -R <hostname>  - Flex rig hostname to listen on (default: %s)\n",
	  DEFAULT_RIG_HOST);
	P("  -p <port>      - Flex rig TCP port to listen on (default: %u)\n",
	  DEFAULT_RIG_PORT);
	P("  -m <model>     - hamlib rig model\n");
	P("  -r <device>    - hamlib rig device\n");
	P("\n");
	P("If neither the hamlib device nor the model are not specified, rig\n"
	  "control will not be available.\n");
	P("\n");
	P("If neither Flex rig hostname nor port are specified, the "
	  "defaults are ignored\n");
	P("and rig control services are not provided.\n");

#undef P

	exit(1);
}

int main(int argc, char **argv)
{
	const char *data_dir;
	const char *rig_host;
	const char *rig_dev;
	uint32_t rig_model;
	uint16_t rig_port;
	int opt;
	int ret;

	setlocale(LC_ALL, "");
	ASSERT0(putenv("TZ=UTC"));

	xthr_set_name("main");

	rig_host = NULL;
	rig_dev = NULL;
	rig_model = 0;
	rig_port = 0;
	while ((opt = getopt(argc, argv, "R:p:m:r:")) != -1) {
		switch (opt) {
			case 'R':
				if (rig_host)
					usage(argv[0], "hamlib rig hostname "
					      "specified multiple times");

				rig_host = optarg;
				break;
			case 'p':
				if (rig_port)
					usage(argv[0], "hamlib rig port "
					      "specified multiple times");

				ret = str2u16(optarg, &rig_port);
				if (ret)
					usage(argv[0], "invalid hamlib rig "
					      "port: %s", xstrerror(ret));

				if (!rig_port)
					usage(argv[0], "invalid hamlib rig "
					      "port: zero port number");
				break;
			case 'm':
				if (rig_model)
					usage(argv[0], "hamlib rig model "
					      "specified multiple times");

				ret = str2u32(optarg, &rig_model);
				if (ret)
					usage(argv[0], "invalid hamlib rig "
					      "model: %s", xstrerror(ret));

				if (!rig_model)
					usage(argv[0], "invalid hamlib rig "
					      "model: zero model number");
				break;
			case 'r':
				if (rig_dev)
					usage(argv[0], "hamlib rig device "
					      "specified multiple times");

				rig_dev = optarg;
				break;
			default:
				usage(argv[0], NULL);
				break;
		}
	}

	/* missing data dir path */
	if ((optind + 1) != argc)
		usage(argv[0], "data directory is required");

	data_dir = argv[argc - 1];

	/*
	 * make sure either both rig model and device are specified, or
	 * neither is specified
	 */
	if ((rig_model || rig_dev) && !(rig_model && rig_dev))
		usage(argv[0], "rig model & device must be both specified");

	/*
	 * make sure both rig host and port are both set if at least one of
	 * them is specified
	 */
	if (rig_host || rig_port) {
		rig_host = rig_host ? rig_host : DEFAULT_RIG_HOST;
		rig_port = rig_port ? rig_port : DEFAULT_RIG_PORT;
	}

	cmn_err(CE_INFO, "hlogd version %s", version_string);
	cmn_err(CE_INFO, "libjeffpc version %s", jeffpc_version);

	/*
	 * chdir to the data directory to make our lives in the future much
	 * easier.  We will be able to use relative paths everywhere.
	 */
	if (chdir(data_dir)) {
		cmn_err(CE_ERROR, "Error: failed to change to data dir: %s",
			strerror(errno));
		goto err;
	}

	ret = load_databases();
	if (ret)
		goto err;

	ret = index_init();
	if (ret)
		goto err;

	ret = announce_start();
	if (ret)
		goto err;

	ret = location_start();
	if (ret)
		goto err_announce;

	ret = rig_start(rig_model, rig_dev, rig_host, rig_port);
	if (ret)
		goto err_location;

	ret = wsjtx_start();
	if (ret)
		goto err_rig;

	for (;;)
		sleep(60);

	wsjtx_stop();
	rig_stop();
	location_stop();
	announce_stop();

	return 0;

err_rig:
	rig_stop();

err_location:
	location_stop();

err_announce:
	announce_stop();

err:
	return 1;
}
